// KKJSetup v1.1 by N's LAB Copyright (C) 2006-2012 by Nobumichi Kozawa
package setup;

import javax.microedition.lcdui.*;
import javax.microedition.midlet.MIDlet;
import javax.microedition.rms.*;

// KKJConfig format
// Byte 0 = Font Size, Kanji edit
//      1 = Font Size, special chracter panel
//          0 = Large, 1 = Mid, 2 = Small
//      2 = Automatic Text Save Option, 0 = Automatic Save, 1= No Save
//      3 bit 0x01 = Freedom mini + Nokia driver mode
//            0x02 = Dictionary Memeory Load (reserved)
//            0x04 = Force SonyEricsson Key Assist
//      4 = User Dictionary, 0 = Not use, 1 = use
//        5 = TextBox menu item prohibition
//        6 = Kanji Input menu item prohibition
//        7 = Input Mode item prohibition
//      8 = E61 Keytop
//      9 = List Conversion Mode count
//     10 = Number of User Dictionary Entries (10-255)
//     11 = (Reserved for User Dictionary Entries)
//     12 = SMS setting
//          0x01 = Automatic FORMETTED_NAME retrieve
//          0x02 = Confirmation message
//          0x04 = Contacts List (original)
//
// KKJShortCut
// int 0 iSpChr = k0;
//     1 iKJModeKey = k1;
//     2 iModeKey = k2;
//     3 iInsKey = k3;
//     4 iAddKey = k4;
//     5 iCopyKey = k5;
//     6 iQuickLoadKey = k6;
//     7 iQuickSaveKey = k7;

public class KKJSetup extends MIDlet implements CommandListener, ItemCommandListener
{
    public Command exitCommand, canCommand, udicCommand, scCommand, scDefCommand, smsCommand, saveCommand;
    public Display disp;
    private List mainList;
    private FontSel fsel;
    private TextSave ts;
    private OtherSel osel;
    private UserDict udic;
    private KFileDialogClass kfd;
    private TbMenu tbmenu;
    private KjMenu kjmenu;
    private IModeMenu imode;
    private E61KeyTop e61key;
    public ShortCut sc;
    private KKeyEdit kke;
    private SmsSetting sms;

    public boolean bE61, bSonyEricsson, bE7_00;
    public byte bKJFont, bSPFont, bTextSave, bOtherFlag, bUserDict, bNumUdic;
    public byte bTbMenuItem, bKjMenuItem, bIMode, bE61key, bListConv, bSms;
    public String sUserDictPath;
    public int iSpChr, iInsKey, iAddKey, iCopyKey, iModeKey, iKJModeKey, iQuickLoadKey, iQuickSaveKey;
    private boolean bSMSInfo = false;

    public KKJSetup() {
        String sPlatform = System.getProperty("microedition.platform");
        //sPlatform = "NokiaE61-1/0618.04.04";
        bE61 = sPlatform.startsWith("NokiaE61");
        bSonyEricsson = sPlatform.startsWith("SonyEricsson");
        bE7_00 = sPlatform.startsWith("NokiaE7-00");
//        bE7_00 = true;

        disp = Display.getDisplay(this);

        String[] stringArray;
        if (bE61) {
            stringArray = new String[]{
                "フォント",
                "テキスト保管",
                "ユーザー辞書",
                "コピペ画面メニュー",
                "漢字入力画面メニュー",
                "文字種",
                "ショートカット",
                "その他の設定",
                "SMS設定",
                "E61キーレイアウト"
            };
        } else {
            stringArray = new String[]{
                "フォント",
                "テキスト保管",
                "ユーザー辞書",
                "コピペ画面メニュー",
                "漢字入力画面メニュー",
                "文字種",
                "ショートカット",
                "その他の設定",
                "SMS設定",
            };
        }

        Image[] imageArray = null;

        mainList = new List("KKJConv設定", Choice.IMPLICIT, stringArray, imageArray);
        exitCommand = new Command("保管", Command.EXIT, 1);
        saveCommand = new Command("保管", Command.ITEM, 1);
        canCommand = new Command("キャンセル", Command.CANCEL, 1);
        mainList.addCommand(exitCommand);
        mainList.addCommand(canCommand);
        mainList.setCommandListener(this);

        fsel = new FontSel();
        fsel.fontForm.addCommand(exitCommand);
        fsel.fontForm.addCommand(canCommand);
        fsel.fontForm.setCommandListener(this);

        ts = new TextSave();
        ts.textForm.addCommand(exitCommand);
        ts.textForm.addCommand(canCommand);
        ts.textForm.setCommandListener(this);

        udic = new UserDict();
        udicCommand = new Command("ファイル選択", Command.ITEM, 1);
        udic.udicForm.addCommand(exitCommand);
        udic.udicForm.addCommand(canCommand);
        udic.siUdicPath.setDefaultCommand(udicCommand);
        udic.siUdicPath.setItemCommandListener(this);
        udic.udicForm.setCommandListener(this);

        osel = new OtherSel();
        osel.otherForm.addCommand(exitCommand);
        osel.otherForm.addCommand(canCommand);
        osel.otherForm.setCommandListener(this);

        tbmenu = new TbMenu();
        tbmenu.tbForm.addCommand(exitCommand);
        tbmenu.tbForm.addCommand(canCommand);
        tbmenu.tbForm.setCommandListener(this);

        kjmenu = new KjMenu();
        kjmenu.kjForm.addCommand(exitCommand);
        kjmenu.kjForm.addCommand(canCommand);
        kjmenu.kjForm.setCommandListener(this);

        imode = new IModeMenu();
        imode.imForm.addCommand(exitCommand);
        imode.imForm.addCommand(canCommand);
        imode.imForm.setCommandListener(this);

        sms = new SmsSetting();
        sms.smsForm.addCommand(exitCommand);
        sms.smsForm.addCommand(canCommand);
        sms.smsForm.setCommandListener(this);
        smsCommand = new Command("保管設定", Command.ITEM, 1);
        sms.siSMSsent.setDefaultCommand(smsCommand);
        sms.siSMSsent.setItemCommandListener(this);

        if (bE61) {
            e61key = new E61KeyTop();
            e61key.e61Form.addCommand(exitCommand);
            e61key.e61Form.addCommand(canCommand);
            e61key.e61Form.setCommandListener(this);
        } else e61key = null;

        sc = new ShortCut();
        scCommand = new Command("キー選択", Command.ITEM, 1);
        scDefCommand = new Command("規定値", Command.ITEM, 2);
        sc.scForm.addCommand(exitCommand);
        sc.scForm.addCommand(canCommand);
        sc.scForm.setCommandListener(sc);
        sc.siSpChr.setDefaultCommand(scCommand);
        sc.siSpChr.setItemCommandListener(this);
        sc.siKanji.setDefaultCommand(scCommand);
        sc.siKanji.setItemCommandListener(this);
        sc.siMode.setDefaultCommand(scCommand);
        sc.siMode.setItemCommandListener(this);
        sc.siIns.setDefaultCommand(scCommand);
        sc.siIns.setItemCommandListener(this);
        sc.siAdd.setDefaultCommand(scCommand);
        sc.siAdd.setItemCommandListener(this);
        sc.siCopy.setDefaultCommand(scCommand);
        sc.siCopy.setItemCommandListener(this);
        sc.siQuickLoad.setDefaultCommand(scCommand);
        sc.siQuickLoad.setItemCommandListener(this);
        sc.siQuickSave.setDefaultCommand(scCommand);
        sc.siQuickSave.setItemCommandListener(this);
        sc.siDefault.setDefaultCommand(scDefCommand);
        sc.siDefault.setItemCommandListener(this);

        loadConfig();
        loadShortCut();
        disp.setCurrent(mainList);
    }

    private void loadShortCut() {
        RecordStore rs = null;
        byte[] ba;

        try {
            rs = RecordStore.openRecordStore("KKJShortCut", false);
            ba = rs.getRecord(1);
            sc.setDefaultValue();
            if (24 == ba.length) {
                iSpChr = ba2int(ba, 0);
                iKJModeKey = ba2int(ba, 1);
                iModeKey = ba2int(ba, 2);
                iInsKey = ba2int(ba, 3);
                iAddKey = ba2int(ba, 4);
                iCopyKey = ba2int(ba, 5);
            } else if (32 == ba.length) {
                iSpChr = ba2int(ba, 0);
                iKJModeKey = ba2int(ba, 1);
                iModeKey = ba2int(ba, 2);
                iInsKey = ba2int(ba, 3);
                iAddKey = ba2int(ba, 4);
                iCopyKey = ba2int(ba, 5);
                iQuickLoadKey = ba2int(ba, 6);
                iQuickSaveKey = ba2int(ba, 7);
            }
            sc.refresh();
        } catch (Exception e) {
            sc.setDefaultValue();
            sc.refresh();
        } finally {
            try {
                if (null != rs) rs.closeRecordStore();
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    private int ba2int(byte[] ba, int pos) {
        int iWork = 0, offset = pos * 4;
        for (int i = offset + 3; i >= offset; i--) {
            iWork *= 128;
            iWork += ba[i];
        }
        return iWork;
    }

    private void loadConfig() {
        RecordStore rsc = null, rsUdicPath = null;
        byte[] ba;

        // Default set
        bKJFont = bSPFont = bE61key = 1;
        bOtherFlag = (byte)((bE7_00) ? 8 : 0);
        bTextSave = bUserDict = bSms = 0;
        bKjMenuItem = bKjMenuDefault;
        bTbMenuItem = bTbMenuDefault;
        bListConv = 3;
        bNumUdic = 100;
        sUserDictPath = (bSonyEricsson) ? "E:/Document/KKJConv/KKJUser.dic" : "E:/KKJConv/KKJUser.dic";

        try {
            rsc = RecordStore.openRecordStore("KKJConfig", false);
            ba = rsc.getRecord(1);
            switch (ba.length) {
                case 12 : bSms = ba[11];
                case 11 : bNumUdic = ba[10];
                case 10 : bListConv = ba[9];
                case 9 : bE61key = ba[8];
                case 8 : bIMode = ba[7];
                case 7 : bKjMenuItem = ba[6];
                case 6 : bTbMenuItem = ba[5];
                case 5 : bUserDict = ba[4];
                case 4 : bOtherFlag = ba[3];
                case 3 : bTextSave = ba[2];
                case 2 : bSPFont = ba[1];
                case 1 : bKJFont = ba[0];
            }
            rsUdicPath = RecordStore.openRecordStore("KKJUdic", false);
            ba = rsUdicPath.getRecord(1);
            sUserDictPath = new String(ba, "UTF-8");
        } catch (Exception e) {
        } finally {
            try {
                if (null != rsc) rsc.closeRecordStore();
                if (null != rsUdicPath) rsUdicPath.closeRecordStore();
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    private void storeShortCut() {
        byte[] ba = new byte[32];

        int2ba(ba, 0, iSpChr);
        int2ba(ba, 1, iKJModeKey);
        int2ba(ba, 2, iModeKey);
        int2ba(ba, 3, iInsKey);
        int2ba(ba, 4, iAddKey);
        int2ba(ba, 5, iCopyKey);
        int2ba(ba, 6, iQuickLoadKey);
        int2ba(ba, 7, iQuickSaveKey);

        try {
            try {
                RecordStore.deleteRecordStore("KKJShortCut");
            } catch (RecordStoreNotFoundException e){
            }
            RecordStore rsc = RecordStore.openRecordStore("KKJShortCut", true);
            rsc.addRecord(ba, 0, ba.length);
            rsc.closeRecordStore();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void int2ba(byte[] ba, int pos, int val) {
        int iWork = val, offset = pos * 4;
        for (int i = offset; i < offset + 4; i++) {
            ba[i] = (byte)(iWork % 128);
            iWork /= 128;
        }
    }

    private void storeConfig() {
        byte[] ba = {
            bKJFont, bSPFont, bTextSave, bOtherFlag, bUserDict, bTbMenuItem, bKjMenuItem, bIMode, bE61key,
            bListConv, bNumUdic, bSms
        };

        try {
            try {
                RecordStore.deleteRecordStore("KKJConfig");
            } catch (RecordStoreNotFoundException e){
            }
            RecordStore rsc = RecordStore.openRecordStore("KKJConfig", true);
            rsc.addRecord(ba, 0, ba.length);
            rsc.closeRecordStore();
            try {
                RecordStore.deleteRecordStore("KKJUdic");
            } catch (RecordStoreNotFoundException e){
            }
            rsc = RecordStore.openRecordStore("KKJUdic", true);
            ba = sUserDictPath.getBytes("UTF-8");
            rsc.addRecord(ba, 0, ba.length);
            rsc.closeRecordStore();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    protected void startApp() {}
    protected void destroyApp(boolean unconditional) {}
    protected void pauseApp() {}

    public void commandAction(Command command, Item item) {
        if (item.equals(udic.siUdicPath)) {
            if (command == udicCommand) {
                kfd = new KFileDialogClass("KKJUser.dic等を選択し保管してください。");
                kfd.fileList.addCommand(saveCommand);
                kfd.fileList.addCommand(canCommand);
                kfd.fileList.setCommandListener(udic);
                kfd.dummyAccess();
                new Thread(new Runnable() {
                    public void run() {
                        kfd.openRoot();
                    }
                }).start();
                   disp.setCurrent(kfd.fileList);
            }
        } else if (command == scDefCommand) {
            sc.setDefault();
        } else if (command == scCommand) {
               if (item.equals(sc.siSpChr)) {
                   sc.pos = 0;
                   kke = new KKeyEdit("記号パネル", this);
             } else if (item.equals(sc.siKanji)) {
                 sc.pos = 1;
                   kke = new KKeyEdit("漢字モード", this);
            } else if (item.equals(sc.siMode)) {
                sc.pos = 2;
                   kke = new KKeyEdit("文字種", this);
               } else if (item.equals(sc.siIns)) {
                   sc.pos = 3;
                   kke = new KKeyEdit("挿入", this);
            } else if (item.equals(sc.siAdd)) {
                sc.pos = 4;
                   kke = new KKeyEdit("追加", this);
               } else if (item.equals(sc.siCopy)) {
                   sc.pos = 5;
                   kke = new KKeyEdit("新規追加", this);
               } else if (item.equals(sc.siQuickLoad)) {
                   sc.pos = 6;
                   kke = new KKeyEdit("クイック読込", this);
               } else if (item.equals(sc.siQuickSave)) {
                   sc.pos = 7;
                   kke = new KKeyEdit("クイック保管", this);
            }
                  kke.addCommand(exitCommand);
               kke.addCommand(canCommand);
               kke.setCommandListener(kke);
               disp.setCurrent(kke);
        } else if (command == smsCommand) {
               Alert smsInfo = null;
            if (!bSMSInfo) {
                   smsInfo = new Alert("SMS送信フォルダー");
                   smsInfo.setTimeout(5000);
                   smsInfo.setType(AlertType.ERROR);
                   smsInfo.setString("SMS送信フォルダー設定内の「保管」はKKJSetupの「保管」とは関係なく単独で設定が保管されます。");
                   bSMSInfo = true;
               }
            SMSSetup smssetup = new SMSSetup(this, sms.smsForm, smsInfo, bSonyEricsson);
            smssetup.disp();
        }
    }

    public void commandAction(Command command, Displayable screen) {
        if (screen.equals(mainList)) {
            if (List.SELECT_COMMAND == command) {
                switch (((List)screen).getSelectedIndex()) {
                    case 0 :
                        fsel.refresh();
                        disp.setCurrent(fsel.fontForm);
                        break;
                    case 1 :
                        ts.refresh();
                        disp.setCurrent(ts.textForm);
                        break;
                    case 2 :
                        udic.refresh();
                        disp.setCurrent(udic.udicForm);
                        break;
                    case 3 :
                        tbmenu.refresh();
                        disp.setCurrent(tbmenu.tbForm);
                        break;
                    case 4 :
                        kjmenu.refresh();
                        disp.setCurrent(kjmenu.kjForm);
                        break;
                    case 5 :
                        imode.refresh();
                        disp.setCurrent(imode.imForm);
                        break;
                    case 6 :
                        sc.refresh();
                        disp.setCurrent(sc.scForm);
                        break;
                    case 7 :
                        osel.refresh();
                        disp.setCurrent(osel.otherForm);
                        break;
                    case 8 :
                        sms.refresh();
                        disp.setCurrent(sms.smsForm);
                        break;
                    case 9 :
                        e61key.refresh();
                        disp.setCurrent(e61key.e61Form);
                        break;
                }
            }
            if (command == exitCommand) {
                storeConfig();
                storeShortCut();
                destroyApp(false);
                notifyDestroyed();
            } else if (command == canCommand) {
                destroyApp(false);
                notifyDestroyed();
            }
        } else if (screen.equals(fsel.fontForm)) {
            if (command == exitCommand) fsel.saveConfig();
            disp.setCurrent(mainList);
        } else if (screen.equals(ts.textForm)) {
            if (command == exitCommand) ts.saveConfig();
            disp.setCurrent(mainList);
        } else if (screen.equals(udic.udicForm)) {
            if (command == exitCommand) udic.saveConfig();
            disp.setCurrent(mainList);
        } else if (screen.equals(tbmenu.tbForm)) {
            if (command == exitCommand) tbmenu.saveConfig();
            disp.setCurrent(mainList);
        } else if (screen.equals(kjmenu.kjForm)) {
            if (command == exitCommand) kjmenu.saveConfig();
            disp.setCurrent(mainList);
        } else if (screen.equals(imode.imForm)) {
            if (command == exitCommand) imode.saveConfig();
            disp.setCurrent(mainList);
        } else if (screen.equals(osel.otherForm)) {
            if (command == exitCommand) osel.saveConfig();
            disp.setCurrent(mainList);
        } else if (screen.equals(sms.smsForm)) {
            if (command == exitCommand) sms.saveConfig();
            disp.setCurrent(mainList);
        } else if (screen.equals(e61key.e61Form)) {
            if (command == exitCommand) e61key.saveConfig();
            disp.setCurrent(mainList);
/*
        } else if (screen.equals(kfd.fileList)) {
            if (List.SELECT_COMMAND == command) {
                   new Thread(new Runnable() {
                       public void run() {
                           kfd.openNext();
                       }
                   }).start();
            } else if (command == exitCommand) {
                String path = kfd.getPathName();
                if (!(path.endsWith("/") || path.endsWith(".."))) {
                    udic.setPathName(path);
                    disp.setCurrent(udic.udicForm);
                }
            } else disp.setCurrent(udic.udicForm);
            */
        }
    }

    // Automatic Text Save Option
    protected class TextSave
    {
        public Form textForm;
        private ChoiceGroup cgTxt;

        public TextSave() {
            if (bTextSave < 0 || bTextSave > 1) bTextSave = 0;
            textForm = new Form("テキスト保管");
            String[] sa = {"YES", "NO"};
            Image[] ia = null;
            cgTxt = new ChoiceGroup("KKJConv終了時にテキストの自動保管を行いますか?", ChoiceGroup.EXCLUSIVE, sa, ia);
            textForm.append(cgTxt);
            refresh();
        }

        public void refresh() {
            cgTxt.setSelectedIndex(bTextSave, true);
        }

        public void saveConfig() {
            bTextSave = (byte)cgTxt.getSelectedIndex();
        }
    }

    // Font Selection Panel
    protected class FontSel
    {
        public Form fontForm;
        private ChoiceGroup cgKj, cgSp;

        public FontSel() {
            if (bKJFont < 0 || bKJFont > 2) bKJFont = 0;
            if (bSPFont < 0 || bSPFont > 2) bSPFont = 0;
            fontForm = new Form("フォントの選択");
            String[] sa = {"大", "中", "小"};
            Image[] ia = null;
            cgKj = new ChoiceGroup("漢字入力画面", ChoiceGroup.EXCLUSIVE, sa, ia);
            cgSp = new ChoiceGroup("記号入力パネル", ChoiceGroup.EXCLUSIVE, sa, ia);
            fontForm.append(cgKj);
            fontForm.append(cgSp);
            refresh();
        }

        public void refresh() {
            cgKj.setSelectedIndex(bKJFont, true);
            cgSp.setSelectedIndex(bSPFont, true);
        }

        public void saveConfig() {
            bKJFont = (byte)cgKj.getSelectedIndex();
            bSPFont = (byte)cgSp.getSelectedIndex();
        }
    }

    protected class UserDict implements CommandListener
    {
        public Form udicForm;
        public StringItem siUdicPath;
        private ChoiceGroup cgUdic;
        private TextField tfUdicPath, tfNumUdic;
        int iUdic, iNumUdic;

        public UserDict() {
            udicForm = new Form("ユーザー辞書");
            String [] sa = {"YES", "NO"};
            Image[] ia = null;
            cgUdic = new ChoiceGroup("ユーザー辞書を使用しますか？", ChoiceGroup.EXCLUSIVE, sa, ia);
            tfUdicPath = new TextField("ユーザー辞書の場所", "", 50, TextField.ANY);
               siUdicPath = new StringItem("ユーザー辞書参照ボタン", "KKJUser.dic等の場所を指定します", Item.BUTTON);
            tfNumUdic = new TextField("ユーザー辞書のエントリー数(1-255)", "", 5, TextField.DECIMAL);
               udicForm.append(cgUdic);
            udicForm.append(siUdicPath);
            udicForm.append(tfUdicPath);
            udicForm.append(tfNumUdic);
        }

        public void setPathName(String name) {
            tfUdicPath.setString(name);
        }

        public void refresh() {
            iUdic = (0 == bUserDict) ? 1 : 0;
            cgUdic.setSelectedIndex(iUdic, true);
            tfUdicPath.setString(sUserDictPath);
            iNumUdic = (bNumUdic < 0) ? 127 - bNumUdic : bNumUdic;
            tfNumUdic.setString(Integer.toString(iNumUdic));
        }

        public void saveConfig() {
            bUserDict = (byte)((0 == cgUdic.getSelectedIndex()) ? 1 : 0);
            sUserDictPath = tfUdicPath.getString();
            try {
                iNumUdic = Integer.parseInt(tfNumUdic.getString());
                if (0 < iNumUdic && iNumUdic < 256) {
                    if (iNumUdic > 127) iNumUdic = 127 - iNumUdic;
                    bNumUdic = (byte) iNumUdic;
                }
            } catch (NumberFormatException e) {}
        }

        public void commandAction(Command command, Displayable screen) {
            if (screen.equals(kfd.fileList)) {
                if (List.SELECT_COMMAND == command) {
                       new Thread(new Runnable() {
                           public void run() {
                               kfd.openNext();
                           }
                       }).start();
                } else if (command == saveCommand) {
                    String path = kfd.getPathName();
                    if (!(path.endsWith("/") || path.endsWith(".."))) {
                        udic.setPathName(path);
                        disp.setCurrent(udic.udicForm);
                    }
                } else disp.setCurrent(udic.udicForm);
            }
        }
    }

    protected class ShortCut implements CommandListener
    {
        public Form scForm;
        public StringItem siKanji, siCopy, siSpChr, siIns, siAdd;
        public StringItem siMode, siDefault, siQuickLoad, siQuickSave;
        public int pos, k0, k1, k2, k3, k4, k5, k6, k7;

        public ShortCut() {
            scForm = new Form("ショートカット定義");
            siSpChr     = new StringItem("記号パネル　", "", Item.BUTTON);
            siKanji     = new StringItem("漢字モード　", "", Item.BUTTON);
            siKanji.setLayout(Item.LAYOUT_NEWLINE_BEFORE);
            siMode      = new StringItem("文字種　　　", "", Item.BUTTON);
            siMode.setLayout(Item.LAYOUT_NEWLINE_BEFORE);
            siIns       = new StringItem("挿入　　　　", "", Item.BUTTON);
            siIns.setLayout(Item.LAYOUT_NEWLINE_BEFORE);
            siAdd       = new StringItem("追加　　　　", "", Item.BUTTON);
            siAdd.setLayout(Item.LAYOUT_NEWLINE_BEFORE);
            siCopy      = new StringItem("新規追加　　", "", Item.BUTTON);
            siCopy.setLayout(Item.LAYOUT_NEWLINE_BEFORE);
            siQuickLoad = new StringItem("クイック読込", "", Item.BUTTON);
            siQuickLoad.setLayout(Item.LAYOUT_NEWLINE_BEFORE);
            siQuickSave = new StringItem("クイック保管", "", Item.BUTTON);
            siQuickSave.setLayout(Item.LAYOUT_NEWLINE_BEFORE);
            siDefault = new StringItem("Default", "規定値に戻す", Item.BUTTON);
            siDefault.setLayout(Item.LAYOUT_NEWLINE_BEFORE);
            scForm.append(siSpChr);
            scForm.append(siKanji);
            scForm.append(siMode);
            scForm.append(siIns);
            scForm.append(siAdd);
            scForm.append(siCopy);
            scForm.append(siQuickLoad);
            scForm.append(siQuickSave);
            scForm.append(siDefault);
            refresh();
        }

        public void setDefault() {
            siSpChr.setText(KKeyEdit.keyName(k0 = 0x2f));
            siKanji.setText(KKeyEdit.keyName(k1 = 11));
            siMode.setText(KKeyEdit.keyName(k2 = 24));
            siIns.setText(KKeyEdit.keyName(k3 = 9));
            siAdd.setText(KKeyEdit.keyName(k4 = 1));
            siCopy.setText(KKeyEdit.keyName(k5 = 3));
            siQuickLoad.setText(KKeyEdit.keyName(k6 = 12));
            siQuickSave.setText(KKeyEdit.keyName(k7 = 19));
        }

        public void setDefaultValue() {
            iSpChr = 0x2f;    // '/'
            iKJModeKey = 11;    // Ctr+K;
            iModeKey = 24;    // Ctr+X
            iInsKey = 9;    // Ctr+I
            iAddKey = 1;    // Ctr+A
            iCopyKey = 3;    // Ctr+C
            iQuickLoadKey = 12; // Ctr+L
            iQuickSaveKey = 19; // Ctr+S
        }

        public void refresh() {
            siSpChr.setText(KKeyEdit.keyName(k0 = iSpChr));
            siKanji.setText(KKeyEdit.keyName(k1 = iKJModeKey));
            siMode.setText(KKeyEdit.keyName(k2 = iModeKey));
            siIns.setText(KKeyEdit.keyName(k3 = iInsKey));
            siAdd.setText(KKeyEdit.keyName(k4 = iAddKey));
            siCopy.setText(KKeyEdit.keyName(k5 = iCopyKey));
            siQuickLoad.setText(KKeyEdit.keyName(k6 = iQuickLoadKey));
            siQuickSave.setText(KKeyEdit.keyName(k7 = iQuickSaveKey));
        }

        public void setValue(int k) {
            switch (pos) {
                case 0 : k0 = k; siSpChr.setText(KKeyEdit.keyName(k)); break;
                case 1 : k1 = k; siKanji.setText(KKeyEdit.keyName(k)); break;
                case 2 : k2 = k; siMode.setText(KKeyEdit.keyName(k)); break;
                case 3 : k3 = k; siIns.setText(KKeyEdit.keyName(k)); break;
                case 4 : k4 = k; siAdd.setText(KKeyEdit.keyName(k)); break;
                case 5 : k5 = k; siCopy.setText(KKeyEdit.keyName(k)); break;
                case 6 : k6 = k; siQuickLoad.setText(KKeyEdit.keyName(k)); break;
                case 7 : k7 = k; siQuickSave.setText(KKeyEdit.keyName(k)); break;
            }
        }

        public void saveConfig() {
            iSpChr = k0;
            iKJModeKey = k1;
            iModeKey = k2;
            iInsKey = k3;
            iAddKey = k4;
            iCopyKey = k5;
            iQuickLoadKey = k6;
            iQuickSaveKey = k7;
        }

        public void commandAction(Command command, Displayable screen) {
            if (screen.equals(scForm)) {
                if (command == exitCommand) saveConfig();
                disp.setCurrent(mainList);
            }
        }
    }

    // Other selection
    protected class OtherSel
    {
        public Form otherForm;
        private ChoiceGroup cgFreedom, cgListConv, cgForceSEKeyAssist, cgStrictShiftMode;
        private int iFreedom, iListConv, iForceSEKeyAssist, iStrictShiftMode;

        public OtherSel() {
            otherForm = new Form("その他の設定");
            String[] sa = {"切", "入"};
            String[] sa2 = {"自動", "入"};
            Image[] ia = null;
            cgListConv = new ChoiceGroup("変換候補のリスト表示", ChoiceGroup.EXCLUSIVE, sa, ia);
            otherForm.append(cgListConv);
            cgFreedom = new ChoiceGroup("Freedom Mini + Nokia Driver 用キーコード変換", ChoiceGroup.EXCLUSIVE, sa, ia);
            otherForm.append(cgFreedom);
            cgForceSEKeyAssist = new ChoiceGroup("SonyEricsson キーボードサポート", ChoiceGroup.EXCLUSIVE, sa2, ia);
            otherForm.append(cgForceSEKeyAssist);
            cgStrictShiftMode = new ChoiceGroup("厳格なシフトモード", ChoiceGroup.EXCLUSIVE, sa, ia);
            otherForm.append(cgStrictShiftMode);
            refresh();
        }

        public void refresh() {
            iListConv = (0 == bListConv) ? 0 : 1;
            cgListConv.setSelectedIndex(iListConv, true);
            iFreedom = (0 == (0x01 & bOtherFlag)) ? 0 : 1;
            cgFreedom.setSelectedIndex(iFreedom, true);
            iForceSEKeyAssist = (0 == (0x4 & bOtherFlag)) ? 0 : 1;
            cgForceSEKeyAssist.setSelectedIndex(iForceSEKeyAssist, true);
            iStrictShiftMode = (0 == (0x8 & bOtherFlag)) ? 0 : 1;
            cgStrictShiftMode.setSelectedIndex(iStrictShiftMode, true);
        }

        public void saveConfig() {
            bListConv = (byte)((0 == cgListConv.getSelectedIndex()) ? 0 : 3);
            bOtherFlag = 0;
            bOtherFlag |= (byte)cgFreedom.getSelectedIndex();
            bOtherFlag |= (byte)cgForceSEKeyAssist.getSelectedIndex() * 4;
            bOtherFlag |= (byte)cgStrictShiftMode.getSelectedIndex() * 8;
        }
    }

    public static byte bTbMenuDefault = 0x0C;

    protected class TbMenu
    {
        public Form tbForm;
        public ChoiceGroup cgTb;

        public TbMenu() {
            tbForm = new Form("コピペ画面メニュー");
            String[] sa = {"SMS送信","文字数","クリアー","コピーto漢字入力画面","Cache Status","Cache Print"};
            Image[] ia = null;
            cgTb = new ChoiceGroup("コピペ画面メニュー選択", ChoiceGroup.MULTIPLE, sa, ia);
            tbForm.append(cgTb);
            refresh();
        }

        public void refresh() {
            cgTb.setSelectedIndex(0, 0 == (0x10 & bTbMenuItem));
            cgTb.setSelectedIndex(1, 0 == (0x01 & bTbMenuItem));
            cgTb.setSelectedIndex(2, 0 == (0x02 & bTbMenuItem));
            cgTb.setSelectedIndex(3, 0 == (0x20 & bTbMenuItem));
            cgTb.setSelectedIndex(4, 0 == (0x04 & bTbMenuItem));
            cgTb.setSelectedIndex(5, 0 == (0x08 & bTbMenuItem));
        }

        public void saveConfig() {
            bTbMenuItem = 0;
            if (!cgTb.isSelected(0)) bTbMenuItem |= 0x10;
            if (!cgTb.isSelected(1)) bTbMenuItem |= 0x01;
            if (!cgTb.isSelected(2)) bTbMenuItem |= 0x02;
            if (!cgTb.isSelected(3)) bTbMenuItem |= 0x20;
            if (!cgTb.isSelected(4)) bTbMenuItem |= 0x04;
            if (!cgTb.isSelected(5)) bTbMenuItem |= 0x08;
        }
    }

    public static byte bKjMenuDefault = 0x4c;

    protected class KjMenu
    {
        public Form kjForm;
        public ChoiceGroup cgKj;

        public KjMenu() {
            kjForm = new Form("漢字入力メニュー");
            String[] sa = {"記号パネル","SMS送信","切替","挿入","追加","新規追加","ファイル","クイックファイル"};
            Image[] ia = null;
            cgKj = new ChoiceGroup("漢字入力メニュー選択", Choice.MULTIPLE, sa, ia);
            kjForm.append(cgKj);
            refresh();
        }

        public void refresh() {
            cgKj.setSelectedIndex(0, 0 == (0x01 & bKjMenuItem));
            cgKj.setSelectedIndex(1, 0 == (0x80 & bKjMenuItem));
            cgKj.setSelectedIndex(2, 0 == (0x02 & bKjMenuItem));
            cgKj.setSelectedIndex(3, 0 == (0x04 & bKjMenuItem));
            cgKj.setSelectedIndex(4, 0 == (0x08 & bKjMenuItem));
            cgKj.setSelectedIndex(5, 0 == (0x10 & bKjMenuItem));
            cgKj.setSelectedIndex(6, 0 == (0x20 & bKjMenuItem));
            cgKj.setSelectedIndex(7, 0 == (0x40 & bKjMenuItem));
        }

        public void saveConfig() {
            bKjMenuItem = 0;
            if (!cgKj.isSelected(0)) bKjMenuItem |= 0x01;
            if (!cgKj.isSelected(1)) bKjMenuItem |= 0x80;
            if (!cgKj.isSelected(2)) bKjMenuItem |= 0x02;
            if (!cgKj.isSelected(3)) bKjMenuItem |= 0x04;
            if (!cgKj.isSelected(4)) bKjMenuItem |= 0x08;
            if (!cgKj.isSelected(5)) bKjMenuItem |= 0x10;
            if (!cgKj.isSelected(6)) bKjMenuItem |= 0x20;
            if (!cgKj.isSelected(7)) bKjMenuItem |= 0x40;
        }
    }

    protected class IModeMenu
    {
        public Form imForm;
        public ChoiceGroup cgIm;

        public IModeMenu() {
            imForm = new Form("文字種");
            String[] sa;
            if (bE61) {
                sa = new String[] {"全英", "半英", "全カナ", "半カナ", "全数記", "半数記"};
            } else {
                sa = new String[] {"全英", "半英", "全カナ", "半カナ"};
            }
            Image[] ia = null;
            cgIm = new ChoiceGroup("文字種選択", Choice.MULTIPLE, sa, ia);
            imForm.append(cgIm);
            refresh();
        }

        public void refresh() {
            cgIm.setSelectedIndex(0, 0 == (0x01 & bIMode));
            cgIm.setSelectedIndex(1, 0 == (0x02 & bIMode));
            cgIm.setSelectedIndex(2, 0 == (0x04 & bIMode));
            cgIm.setSelectedIndex(3, 0 == (0x08 & bIMode));
            if (bE61) {
                cgIm.setSelectedIndex(4, 0 == (0x10 & bIMode));
                cgIm.setSelectedIndex(5, 0 == (0x20 & bIMode));
            }
        }

        public void saveConfig() {
            bIMode = 0;
            if (!cgIm.isSelected(0)) bIMode |= 0x01;
            if (!cgIm.isSelected(1)) bIMode |= 0x02;
            if (!cgIm.isSelected(2)) bIMode |= 0x04;
            if (!cgIm.isSelected(3)) bIMode |= 0x08;
            if (bE61) {
                if (!cgIm.isSelected(4)) bIMode |= 0x10;
                if (!cgIm.isSelected(5)) bIMode |= 0x20;
            }
        }
    }

    protected class E61KeyTop
    {
        public Form e61Form;
        private ChoiceGroup cgE61;
        private int iE61key;

        public E61KeyTop() {
            e61Form = new Form("E61キーレイアウト");
            String[] sa = {"　[!][&][-][1][2][3][*][/][=][?]", "　[!][\"][&][1][2][3][*][-][+][=]", "中[!][\"][&][1][2][3][*][-][+][=]", "　[!][\"][-][1][2][3][*][+][=][?]", "台[ ][ ][ ][1][2][3][*][ ][ ][ ]"};
            Image[] ia = null;
            cgE61 = new ChoiceGroup("E61記号キーの選択", ChoiceGroup.EXCLUSIVE, sa, ia);
            e61Form.append(cgE61);
            refresh();
        }

        public void refresh() {
            iE61key = bE61key - 1;
            if (0 > iE61key || iE61key > 4) iE61key = 0;
            cgE61.setSelectedIndex(iE61key, true);
        }

        public void saveConfig() {
            bE61key = (byte)(cgE61.getSelectedIndex() + 1);
        }
    }

    protected class SmsSetting
    {
        public Form smsForm;
        private ChoiceGroup cgAutoName, cgConfirm, cgContacts;
        public StringItem siSMSsent;
        private int iAutoName, iConfirm, iContacts;

        public SmsSetting() {
            smsForm = new Form("SMSの設定");
            String[] sa = {"入", "切"};
            Image[] ia = null;
            siSMSsent = new StringItem("SMS送信済みフォルダー", "SMS保管設定", Item.BUTTON);
            siSMSsent.setLayout(Item.LAYOUT_NEWLINE_BEFORE);
            smsForm.append(siSMSsent);
            cgAutoName = new ChoiceGroup("SMS発信時の自動名前検索", ChoiceGroup.EXCLUSIVE, sa, ia);
            smsForm.append(cgAutoName);
            cgConfirm = new  ChoiceGroup("SMS発信完了メッセージ", ChoiceGroup.EXCLUSIVE, sa, ia);
            smsForm.append(cgConfirm);
            cgContacts = new ChoiceGroup("オリジナル電話帳", ChoiceGroup.EXCLUSIVE, sa, ia);
            smsForm.append(cgContacts);
            refresh();
        }

        public void refresh() {
            iAutoName = (0 == (0x01 & bSms)) ? 0 : 1;
            cgAutoName.setSelectedIndex(iAutoName, true);
            iConfirm = (0 == (0x02 & bSms)) ? 0 : 1;
            cgConfirm.setSelectedIndex(iConfirm, true);
            iContacts = (0 == (0x04 & bSms)) ? 0 : 1;
            cgContacts.setSelectedIndex(iContacts, true);
        }

        public void saveConfig() {
            bSms = 0;
            bSms |= (byte)cgAutoName.getSelectedIndex();
            bSms |= (byte)cgConfirm.getSelectedIndex() * 2;
            bSms |= (byte)cgContacts.getSelectedIndex() * 4;
        }
    }
}
